ihe = {}

require 'ihe.xml'
require 'ihe.help'

ihe.id = {}
ihe.value = {}
ihe.code = {}
ihe.code.simple = {}
ihe.text = {}
ihe.interval = {}
ihe.name = {}
ihe.address = {}

local addElement = node.addElement
local setAttr = node.setAttr
local setInner = node.setInner
local setText = node.setText

ControlActProcessMoodCodes = {
   ['event']='EVN',
   ['intent']='INT',
   ['request']='RQO',
   ['proposal']='PRP',
   ['recommendation']='RMD',
   ['appointment']='APT',
   ['appointment request']='ARQ',   
   ['promise']='PRMS'
}

local GetUUIDHelp = {
   title="ihe.getUUID",
   usage="ihe.getUUID()",
   description="This function will return a guid.",
   returns={
      "The guid"
   },
   parameters={},
   examples={"local GUID = ihe.getUUID()"}
}

function ihe.getUUID()
   return util.guid(128)
end

ihe.help.set{func=ihe.getUUID, info=GetUUIDHelp}

local IdAddHelp = {
   title="ihe.id.add",
   usage="ihe.id.add{target=<target>, element=<value>, root=<value>, extension=<value>}",
   description="This function will add a id construct to the xml document.",
   returns={
      "The node tree presenting the resulting xml construct"
   },
   parameters={
      target='The parent element of the id',
      element='The name of the element storing the id',
      root='The root attribute of the id'
   },
   optional_parameters={
      extension='The extension attribute of the id'
   },
   examples={"local IdXML = ihe.id.add{target=Root, element='id', root='2220c1c4-87ef-11dc-b865-3603d6866807'}"}
}

function ihe.id.add(I)
   ihe.help.check(I, IdAddHelp)
   
   local ID = addElement(I.target, I.element)
   setAttr(ID, 'root', I.root)
   if I.extension then
      setAttr(ID, 'extension', I.extension)
   end
   
   return ID
end

ihe.help.set{func=ihe.id.add, info=IdAddHelp}

local ValueAddHelp = {
   title="ihe.value.add",
   usage="ihe.value.add{target=<target>, element=<value>, value=<value>}",
   description="This function will add a value construct to the xml document.",
   returns={
      "The node tree presenting the resulting xml construct"
   },
   parameters={
      target='The parent element of the value',
      element='The element storing the value',
      value='The value'
   },
   optional_parameters={
      type='The value type eg. INT'
   },
   examples={"local ValueXML = ihe.value.add{target=Root, element='creationTime', value='20130820165800'}"}
}

function ihe.value.add(I)
   ihe.help.check(I, ValueAddHelp)
   
   local V = addElement(I.target, I.element)
   if I.type then
      setAttr(V, 'xsi:type', I.type)
   end
   setAttr(V, 'value', I.value)
   
   return V
end

ihe.help.set{func=ihe.value.add, info=ValueAddHelp}

local NameAddHelp = {
   title="ihe.name.add",
   usage=[[ihe.name.add{target=<target>, element=<value>, given=<value>, 
   family=<value>, middle=<value>, title=<value>, suffix=<value>}]],
   description="This function will add a name construct to the xml document.",
   returns={
      "The node tree presenting the resulting xml construct"
   },
   parameters={
      target='The parent element of the name element',
      element='The element storing the name',
      given='The given name',
      family='The family name'
   },
   optional_parameters={
      delimiter='The delimiter',
      prefix='The title eg. Mrs.',
      suffix='The suffix eg. Ph.D'
   },
   examples={[[local NameXML = ihe.name.add{target=Root, element='value', 
      given='Jimmy', family='Jones'}]]}
}

function ihe.name.add(I)
   ihe.help.check(I, NameAddHelp)
   
   local N = addElement(I.target, I.element)
   ihe.text.add{target=N, element='given', text=I.given}
   ihe.text.add{target=N, element='family', text=I.family}
   if I.prefix then
      ihe.text.add{target=N, element='prefix', text=I.prefix}
   end 
   if I.suffix then
      ihe.text.add{target=N, element='suffix', text=I.suffix}
   end    
   if I.delimiter then
      ihe.text.add{target=N, element='delimiter', text=I.delimiter}
   end
   if I.use then
      setAttr(N, 'use', I.use)
   end
   
   return N
end

ihe.help.set{func=ihe.name.add, info=NameAddHelp}

local AddressAddHelp = {
   title="ihe.address.add",
   usage=[[ihe.address.add{target=<target>, element=<value>, street=<value>, 
   city=<value>, state=<value>, zip=<value>}]],
   description="This function will add a address construct to the xml document.",
   returns={
      "The node tree presenting the resulting xml construct"
   },
   parameters={
      target='The parent element of the address element',
      element='The element storing the address',
      street='The residence number and street name eg. 100 Park Ave.',
      city='The city',
      state='The state'
   },
   optional_parameters={
      zip='The zip'
   },
   examples={[[local AddressXML = ihe.address.add{target=Root, element='value', 
      street='10 Street Rd.', city='Gotham', state='New York'}]]}
}

function ihe.address.add(I)
   ihe.help.check(I, AddressAddHelp)
   
   local A = addElement(I.target, I.element)
   ihe.text.add{target=A, element='street', text=I.street}
   ihe.text.add{target=A, element='city', text=I.city}
   ihe.text.add{target=A, element='state', text=I.state}
   if I.zip then
      ihe.text.add{target=A, element='zip', text=I.zip}
   end    
   
   return A
end

ihe.help.set{func=ihe.address.add, info=AddressAddHelp}

local IntervalAddHelp = {
   title="ihe.interval.add",
   usage="ihe.interval.add{target=<target>, element=<value>, low=<value>, high=<value>}",
   description="This function will add an interval construct to the xml document.",
   returns={
      "The node tree presenting the resulting xml construct"
   },
   parameters={
      target='The parent element of the interval',
      element='The element storing the interval'
   },
   optional_parameters={
      low='The start of the interval',
      high='The end of the interval'
   },
   examples={"local ValueXML = ihe.interval.add{target=Root, element='effectiveTime', low='20130820125800', high='20130820165800'}"}
}

function ihe.interval.add(I)
   ihe.help.check(I, IntervalAddHelp)
   
   local V = addElement(I.target, I.element)
   if I.low then
      ihe.value.add{target=V, element='low', value=I.low}
   end
   if I.high then
      ihe.value.add{target=V, element='high', value=I.high}
   end
   
   return V
end

ihe.help.set{func=ihe.interval.add, info=IntervalAddHelp}

local CodeAddHelp = {
   title="ihe.code.add",
   usage="ihe.code.add{target=<target>, element=<value>, code=<value>, system=<value>, displayName=<value>, systemName=<value>}",
   description="This function will add a coded construct to the xml document.",
   returns={
      "The node tree presenting the resulting xml construct"
   },
   parameters={
      target='The parent element of the coded element',
      element='The element storing the coded value',
      code='The code',
      system='The system where the code can be found'
   },
   optional_parameters={
      displayName='The display name of code',
      systemName='The name of the system containing the code'
   },
   examples={"local CodeXML = ihe.code.add{target=Root, element='code', code='PRPA_TE201309UV02', system='2.16.840.1.113883.1.6'}"}
}

function ihe.code.add(I)
   ihe.help.check(I, CodeAddHelp)
   
   local C = addElement(I.target, I.element)   
   setAttr(C, 'code', I.code)
   setAttr(C, 'codeSystem', I.system)
   
   if I.displayName then
      setAttr(C, 'displayName', I.displayName)
   end   
   if I.systemName then
      setAttr(C, 'systemName', I.systemName)
   end
   
   return C   
end

ihe.help.set{func=ihe.code.add, info=CodeAddHelp}

local CodeSimpleAddHelp = {
   title="ihe.code.simple.add",
   usage="ihe.code.simple.add{target=<target>, element=<value>, code=<value>}",
   description="This function will add a simple coded construct to the xml document.",
   returns={
      "The node tree presenting the resulting xml construct"
   },
   parameters={
      target='The parent element of the coded element',
      element='The element storing the coded value',
      code='The code'
   },
   examples={"local CodeSimpleXML = ihe.code.simple.add{target=Root, element='versionCode', code='V3PR1'}"}
}

function ihe.code.simple.add(I)
   ihe.help.check(I, CodeSimpleAddHelp)
   
   local C = addElement(I.target, I.element)   
   setAttr(C, 'code', I.code)

   return C   
end

ihe.help.set{func=ihe.code.simple.add, info=CodeSimpleAddHelp}

local TextAddHelp = {
   title="ihe.text.add",
   usage="ihe.text.add{target=<target>, element=<value>, text=<value>}",
   description="This function will add a text construct to the xml document.",
   returns={
      "The node tree presenting the resulting xml construct"
   },
   parameters={
      target='The parent element of the text element',
      element='The element storing the text (as its inner content)',
      text='The text'
   },
   examples={"local TextXML = ihe.text.add{target=Root, element='semanticsText', text='Patient.id'}"}
}

function ihe.text.add(I)
   ihe.help.check(I, TextAddHelp)
   
   local T = addElement(I.target, I.element)
   setText(T, I.text)
   
   return T
end

ihe.help.set{func=ihe.text.add, info=TextAddHelp}
