require 'xds_b.pnr.code'

xds_b.pnr.documentEntry = {}
xds_b.pnr.documentEntry.id = {}
xds_b.pnr.documentEntry.code = {}
xds_b.pnr.documentEntry.author = {}
xds_b.pnr.documentEntry.association = {}

local addElement = node.addElement
local setAttr = node.setAttr

local DocumentEntryAddHelp = {
   title="xds_b.pnr.documentEntry.add",
   usage="xds_b.pnr.documentEntry.add{target=<root tag>, id=<value>, mimeType=<value>}",
   description="This function will add a documentEntry construct to the xml document.",
   returns={
      "The node tree presenting the resulting xml construct"
   },
   parameters={
      target="The root xml where the documentEntry XML will be inserted",
      documentEntryId="The documentEntry Id",
      mime="The document content type"
   },
   examples={"local DocumentEntryXML = xds_b.pnr.documentEntry.add{target=Root, id='DocumentEntry1', mime='text/plain'}"}
}

function xds_b.pnr.documentEntry.add(I)
   ihe.help.check(I, DocumentEntryAddHelp)
   
   local EO = addElement(I.target, 'rim:ExtrinsicObject')
   setAttr(EO, 'id', I.documentEntryId)
   setAttr(EO, 'mimeType', I.mime)  
   setAttr(EO, 'objectType', 'urn:uuid:7edca82f-054d-47f2-a032-9b2a5b5186c1')
   
   return EO
end

ihe.help.set{func=xds_b.pnr.documentEntry.add, info=DocumentEntryAddHelp}

local DocumentEntryAuthorAddHelp = {
   title="xds_b.pnr.documentEntry.author.add",
   usage="xds_b.pnr.documentEntry.author.add{target=<root tag>, documentEntryId=<value>, id=<value>}",
   description="This function will add an author construct to the xml document.",
   returns={
      "The node tree presenting the resulting xml construct"
   },
   parameters={
      target="The document entry root xml where the author XML will be inserted",
      documentEntryId="The document entry Id",
      id="The Id associated with the author XML"
   },
   examples={[[local AuthorXML = xds_b.pnr.documentEntry.author.add{target=DocumentEntryRoot, documentEntryId='DocumentEntry1', 
      id='id_45'}]]}
}

function xds_b.pnr.documentEntry.author.add(I)
   ihe.help.check(I, DocumentEntryAuthorAddHelp)
   
   local C = addElement(I.target, 'rim:Classification')
   setAttr(C, 'classificationScheme', 'urn:uuid:93606bcf-9494-43ec-9b4e-a7748d1a838d')
   setAttr(C, 'classifiedObject', I.documentEntryId)
   setAttr(C, 'nodeRepresentation', '')
   setAttr(C, 'objectType', 'urn:oasis:names:tc:ebxml-regrep:ObjectType:RegistryObject:Classification')
   setAttr(C, 'id', I.id)
   
   return C
end

ihe.help.set{func=xds_b.pnr.documentEntry.author.add, info=DocumentEntryAuthorAddHelp}

local DocumentEntryIdAddHelp = {
   title="xds_b.pnr.documentEntry.id.add",
   usage="xds_b.pnr.documentEntry.id.add{target=<root tag>, idType=<value>, id=<value>, value=<value>}",
   description="This function will add an id construct to the xml document.",
   returns={
      "The node tree presenting the resulting xml construct"
   },
   parameters={
      target="The document entry root xml where the id XML will be inserted",
      idType="The kind of Id i.e. patient or unique",
      id="The Id associated with the Id within the request",
      documentEntryId="The Id of the document entry enclosing the id",
      value="The value associated with the Id XML"
   },
   examples={[[local IdXML = xds_b.pnr.documentEntry.id.add{target=DocumentEntryRoot, idType='unique', id='id_55', 
      value='1.2009.0827.08.33.501'}]]}
}

function xds_b.pnr.documentEntry.id.add(I)
   ihe.help.check(I, DocumentEntryIdAddHelp)
   
   local EI = addElement(I.target, 'rim:ExternalIdentifier')
   setAttr(EI, 'identificationScheme', '')
   setAttr(EI, 'value', I.value)
   setAttr(EI, 'objectType', 'urn:oasis:names:tc:ebxml-regrep:ObjectType:RegistryObject:ExternalIdentifier')
   setAttr(EI, 'id', I.id)
   setAttr(EI, 'registryObject', I.documentEntryId)
   
   if 'unique' == I.idType then
      EI.identificationScheme = 'urn:uuid:2e82c1f6-a085-4c72-9da3-8640a32e42ab'  
      xds_b.pnr.name.add{target=EI, name='XDSDocumentEntry.uniqueId'}
   elseif 'patient' == I.idType then
      EI.identificationScheme = 'urn:uuid:58a6f841-87b3-4a3e-92fd-a8ffeff98427' 
      xds_b.pnr.name.add{target=EI, name='XDSDocumentEntry.patientId'}
   end
  
   return EI
end

ihe.help.set{func=xds_b.pnr.documentEntry.id.add, info=DocumentEntryIdAddHelp}

local DocumentEntryAssociationAddHelp = {
   title="xds_b.pnr.documentEntry.association.add",
   usage=[[xds_b.pnr.documentEntry.association.add{target=<root tag>, submissionSetId=<value>, 
   documentEntryId=<value>, id=<value>, status=<value>}]],
   description="This function will add an association construct to the xml document.",
   returns={
      "The node tree presenting the resulting xml construct"
   },
   parameters={
      target="The root xml where the association XML will be inserted",
      submissionSetId="The submission set Id to associate",
      documentEntryId="The document entry Id to associate",
      id="The Id associated with the association within the request",
      status="The type of submission"
   },
   examples={[[local AssociationXML = xds_b.pnr.documentEntry.association.add{target=Root, submissionSetId='SubmissionSet1', 
      documentEntryId='DocumentEntry1', id='id_95', status='Original'}]]}
}

function xds_b.pnr.documentEntry.association.add(I)
   ihe.help.check(I, DocumentEntryAssociationAddHelp)
   
   local AS = addElement(I.target, 'rim:Association')  
   setAttr(AS, 'associationType', 
      'urn:oasis:names:tc:ebxml-regrep:AssociationType:HasMember')
   setAttr(AS, 'sourceObject', I.submissionSetId)
   setAttr(AS, 'targetObject', I.documentEntryId)
   setAttr(AS, 'id', I.id)
   
   xds_b.pnr.slot.add{target=AS, name='SubmissionSetStatus', values={I.status}}
   
   return AS
end

ihe.help.set{func=xds_b.pnr.documentEntry.association.add, info=DocumentEntryAssociationAddHelp}